"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeInputFilter = void 0;

/*
 * Predefined input filter by Valium
 */
var VALIUM_INPUT_FILTERS = {
  'int': /^-?\d+$/,
  'uint': /^\d+$/,
  'float': /^-?\d*[.,]?\d*$/,
  //'float'       : /[+-]?([0-9]*[.])?[0-9]+/,
  'dollar': /^-?\d*[.,]?\d{0,2}$/,
  //'euro'        : /^-?\d*[.,]?\d{0,2}$/,
  'latin': /^[a-z ]*$/i,
  'hexadecimal': /^[0-9a-f]*$/i
};

var _fltBase = regex => {
  return v => {
    if (v === undefined || v === '') {
      return true;
    }

    return regex.test(v);
  };
};

var makeInputFilter = (inputFilter, inputName) => {
  if (inputFilter == undefined || inputFilter === '') {
    return undefined;
  }

  if (typeof inputFilter === 'string') {
    var regex = VALIUM_INPUT_FILTERS[inputFilter];

    if (regex === undefined) {
      console.error("Valium: error on Input Element (".concat(inputName, "). (").concat(inputFilter, ") is not a valid inputFilter"));
      return undefined;
    }

    return _fltBase(regex);
  }

  if (inputFilter instanceof RegExp) {
    return _fltBase(inputFilter);
  }

  if (typeof inputFilter === "function") {
    return inputFilter;
  }

  console.error("Valium: error on Input Element (".concat(inputName, "). (").concat(inputFilter, ") of type (").concat(typeof inputFilter, ") is not a valid inputFilter"));
  return undefined;
};

exports.makeInputFilter = makeInputFilter;