"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useInputFilter = void 0;

var _react = require("react");

var _makeInputFilter = require("./makeInputFilter");

// This event list would cover every need:
// ['input', 'keydown', 'keyup', 'mousedown', 'mouseup', 'select', 'contextmenu', 'drop'],
// But lets start simple and easy. 
var INPUT_FILTER_EVENT_TYPES = ['input', 'keydown', 'mousedown'];

var useInputFilter = (inputRef, inputFilter) => {
  (0, _react.useEffect)(() => {
    if (inputFilter == undefined) {
      return;
    }

    var input = inputRef === null || inputRef === void 0 ? void 0 : inputRef.current;

    if (input == undefined) {
      return;
    }

    if (input.type.toLowerCase() != 'text') {
      return;
    }

    var allListeners = {}; // Input Filter listeners
    // Credits to:
    // https://stackoverflow.com/a/469362
    // https://jsfiddle.net/emkey08/zgvtjc51

    var theInputFilter = (0, _makeInputFilter.makeInputFilter)(inputFilter, input.name); // init auxiliar properties

    input.oldValue = input.value;

    var filterEventListener = function filterEventListener(event) {
      if (theInputFilter(event.target.value)) {
        event.target.oldValue = event.target.value;
      } else if (Object.hasOwnProperty.call(event.target, "oldValue")) {
        var selectionStart = event.target.selectionStart;
        var selectionEnd = event.target.selectionEnd;
        event.target.value = event.target.oldValue;

        try {
          event.target.setSelectionRange(selectionStart - 1, selectionEnd - 1);
        } catch (e) {}
      } else {
        event.target.value = "";
      }
    };

    INPUT_FILTER_EVENT_TYPES.forEach(function (eventType) {
      input.addEventListener(eventType, filterEventListener);
      allListeners[eventType] = filterEventListener;
    }); // clean listeners function

    var removeAllChangeListeners = () => {
      if (input != undefined) {
        Object.keys(allListeners).map(eventType => {
          input.removeEventListener(eventType, allListeners[eventType]);
        });
      }
    }; // return clean function


    return removeAllChangeListeners;
  }, [inputRef, inputFilter]);
};

exports.useInputFilter = useInputFilter;