"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useForm = void 0;

var _react = require("react");

var _log = require("./helpers/log");

var useForm = () => {
  var formRef = (0, _react.useRef)(undefined);
  var [valid, setValid] = (0, _react.useState)(true);
  var readElements = (0, _react.useCallback)(() => {
    if ((formRef === null || formRef === void 0 ? void 0 : formRef.current) == undefined) {
      return {};
    }

    var formElements = formRef.current.elements;

    if (!formElements) {
      return {};
    }

    var nElements = {};

    for (var idx = 0; idx < formElements.length; idx++) {
      var el = formElements.item(idx);
      var msg = el.getAttribute('data-valium-validity') || '';
      var val = el.getAttribute('data-valium-value') || '';

      var _valid = msg == '';

      nElements[el.name] = {
        valid: _valid,
        message: msg,
        value: val
      };
    }

    return nElements;
  }, []);
  var checkIsValid = (0, _react.useCallback)(() => {
    var isValid = true;

    for (var v of Object.values(readElements())) {
      if (v.valid === false) {
        isValid = false;
        break;
      }
    }

    return isValid;
  }, [readElements]);
  (0, _react.useEffect)(() => {
    if ((formRef === null || formRef === void 0 ? void 0 : formRef.current) == undefined) {
      return;
    }

    try {
      formRef.current.noValidate = true;
    } catch (e) {
      console.error(e);
    } // Initial check of form validity


    var isValid = checkIsValid();
    (0, _log.log)('form', "useEffect(1) Setting valid to ".concat(isValid));
    setValid(isValid);
  }, [checkIsValid]);
  (0, _react.useEffect)(() => {
    if ((formRef === null || formRef === void 0 ? void 0 : formRef.current) == undefined) {
      return;
    }

    var formValidityListener = event => {
      var elName = event.detail.name; //const elValid= event.detail.valid
      //const validity= event.detail.validity

      var value = event.detail.value;
      var nValid = checkIsValid();
      (0, _log.log)('form', "on ".concat(event.type, " ").concat(elName, ". Valid? ").concat(nValid, " (curr ").concat(valid, ") (value ").concat(value, ")"));

      if (valid != nValid) {
        setValid(nValid);
      }
    };

    formRef.current.addEventListener('valium-form-change', formValidityListener); // clean listeners function

    var removeAllChangeListeners = () => {
      formRef.current.removeEventListener('valium-form-change', formValidityListener);
    };

    (0, _log.log)('form', "useEffect(2) Listeners prepared");
    return removeAllChangeListeners;
  }, [checkIsValid, valid]);
  (0, _log.log)('form', "Render, valid is ".concat(valid));
  return [formRef, valid, readElements];
};

exports.useForm = useForm;