"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useValidationHandler = void 0;

var _react = require("react");

var _log = require("../helpers/log");

var _checkValidity = require("./checkValidity");

var _getInputValue = require("../config/getInputValue");

var _getEventTarget = require("../config/getEventTarget");

var _getDefaultMessage = require("../config/getDefaultMessage");

var useValidationHandler = (transformValue, checkValue, allowedValues, disallowedValues, doRepeat, doNotRepeat, decimals, feedback) => {
  var [validity, setValidity] = (0, _react.useState)(false);
  var handler = (0, _react.useCallback)((event, inputRef) => {
    var input = event ? (0, _getEventTarget.getEventTarget)(event) : inputRef === null || inputRef === void 0 ? void 0 : inputRef.current;
    var nValidity = '';

    if (input != undefined) {
      // Clear previous custom error
      input.setCustomValidity('');
      input.setAttribute('data-valium-validity', '');
      input.removeAttribute('data-valium-value'); // Get input value

      var value = (0, _getInputValue.getInputValue)(input);

      if (transformValue != undefined) {
        value = transformValue(value);
      } // Check validity


      var chkValidity = (0, _checkValidity.checkValidity)(input, value, checkValue, allowedValues, disallowedValues, doRepeat, doNotRepeat, decimals);
      nValidity = chkValidity == '' ? '' : feedback != undefined ? feedback : (0, _getDefaultMessage.getDefaultMessage)(chkValidity); // Set it
      //input.removeAttribute('readonly')

      input.setCustomValidity(nValidity);
      input.setAttribute('data-valium-validity', nValidity);
      input.setAttribute('data-valium-value', value); // Update form     

      if (input.form != undefined) {
        //log('input', `${input.name} (${input.type}) #${input.id} validationHandler() is raising valium-form-change`)
        var _event = new CustomEvent("valium-form-change", {
          detail: {
            name: input.name,
            validity: nValidity,
            valid: nValidity == '',
            value: value
          }
        });

        input.form.dispatchEvent(_event);
      }

      (0, _log.log)('input', "".concat(input.name, " (").concat(input.type, ") #").concat(input.id, " validationHandler() new validity is -").concat(nValidity || 'ok', "-"));
    }

    setValidity(nValidity);
  }, [transformValue, checkValue, allowedValues, disallowedValues, doRepeat, doNotRepeat, decimals, feedback]);
  return [validity, handler];
};

exports.useValidationHandler = useValidationHandler;